%% Script for trying some models on coupled EEM NMR LC-MS data

%------------------------------Chemo_simu -----------------------------%
% This file tests the Coupled CP ALS algorithms on Data by Acar et al.
%[E. Acar, E. E. Papalexakis, G. Gurdeniz,  M. A. Rasmussen,  A. J. Lawaetz,...
% M. Nilsson,  R. Bro, Structure-Revealing Data Fusion, BMC Bioinformatics, 15: 239, 2014.]
%
% Comments : Compression for fluorescence data

% External functions:
%
% - kr.m              : Khatri-Rao product;
% - als.m             : ALS algorithm;
% - amb_correct_1f.m  : Matching Permutation for two CP models based on 
%                       factors C only;
% - amb_correct.m     : Matching Permutation for two CP models;
% - ccp_als.m         : Coupled CP ALS.
%
% List of updates                 -     04/02/2016  -     J.E. Cohen 
%                                       Creation of the file
%-------------------------------------------------------------------------%

%--------------Cleaning the workspace and loading functions---------------%
clc;
clear     all;
close     all;
%-------------------------------------------------------------------------%

%load('/Users/cohenjer/Documents/Travail/MATLAB/Data/Data Chemo/data/Multimodal/EEM_NMR_LCMS.mat')
load('D:/Travail/Travail/MATLAB/Data/Data Chemo/data/Multimodal/EEM_NMR_LCMS.mat')

%------------------------ Missing values preprocessing -------------------%

Tx        =    X.data;
Tx        =    permute(Tx,[3,2,1]);

% 3-way block dimensions
dim_X     =     size(Tx);

Tx        =    reshape(Tx,prod(dim_X),1);
temp_x         =     Tx(isnan(Tx));
%temp_miss      =     isnan(Tx);
Tx(isnan(Tx))  =     abs(0.05*randn(size(temp_x)));
Tx             =     reshape(Tx,dim_X);
normTx         =     (Tx(:))'*(Tx(:));
Tx             =     Tx/sqrt(normTx); 
%-----------------------------Model parameters----------------------------%
% Number of components
R_X       =     3;
% Standard deviations of the measurements noise
sigma_n   =     1;
% Number of iterations
iter      =     1000;
% Number of initialization
N    =    1;

%-------------------------------------------------------------------------%

%----------------------- Tucker3 Coupled Compression ---------------------%

Tx_m      =     reshape( permute( Tx , [3,1,2] ) , dim_X(3) , dim_X(2)*dim_X(1) );
dim1c     =     [6,6,6];%[3,3,3];
dim2c     =     [21,20,28];%


[Gx,U,V,W,~,~,Sigma_1]     =     hosvd2(Tx,dim1c,'random');
[Gx2,U2,V2,W2]             =     hosvd(Tx,dim2c,'random');

%figure, plot(diag(Sigma_1),'b')
%figure, plot(log(diag(Sigma_1)),'b')

%------------------------------Computation--------------------------------%

Stockerr1   =     inf*ones(1,iter);
Stockerr2   =     inf*ones(1,iter);
Stockerr3   =     inf*ones(1,iter);
Stockerr4   =     inf*ones(1,iter);

Stocktps1   =     inf*ones(1,iter);
Stocktps2   =     inf*ones(1,iter);
Stocktps3   =     inf*ones(1,iter);
Stocktps4   =     inf*ones(1,iter);

for n=1:N

%Tx        =    reshape(Tx,prod(dim_X),1);
%temp_x         =     Tx(isnan(Tx));
%Tx(isnan(Tx))  =     abs(0.05*randn(size(temp_x)));
%Tx             =     reshape(Tx,dim_X);
%normTx         =     (Tx(:))'*(Tx(:));
%Tx             =     Tx/sqrt(normTx);     
    
%-----------------------------Initialization------------------------------%
% First tensor
A0         =     abs(randn(dim_X(1),R_X));
B0         =     abs(randn(dim_X(2),R_X));
C0         =     abs(randn(dim_X(3),R_X));
A0         =     A0.*repmat(1./sqrt(sum(A0.^2)),dim_X(1),1);
B0         =     B0.*repmat(1./sqrt(sum(B0.^2)),dim_X(2),1); 
C0         =     C0.*repmat(1./sqrt(sum(C0.^2)),dim_X(3),1); 
%-------------------------------------------------------------------------%

%---------------------------Uncoupled ALS---------------------------------%
% ALS
[Aals, Bals, Cals, err_als,temps_als]         =     als(Tx,iter,A0,B0,C0);

% ANLS
[Aanls, Banls, Canls, err_anls,temps_anls]    =     anls(Tx,iter,A0,B0,C0);

% Compressed ALS
[Apc,Bpc,Cpc,err_pc,temps_pc,eta]     =     proco_als(Tx,iter,dim1c,'standard','given',A0,B0,C0,0,Gx,U,V,W);

% Compressed ALS big
[Apc2,Bpc2,Cpc2,err_pc2,temps_pc2]     =     proco_als(Tx,iter,dim2c,'standard','given',A0,B0,C0,0,Gx2,U2,V2,W2);



% Error Comparaison and keeping best run
if err_pc(end)<Stockerr1(end)
    Apcs   =    Apc;Bpcs   =    Bpc;Cpcs   =    Cpc;
    Stockerr1=err_pc;
    Stocktps1=temps_pc;
end

if err_pc2(end)<Stockerr4(end)
    Apcs2   =    Apc2;Bpcs2   =    Bpc2;Cpcs2   =    Cpc2;
    Stockerr4=err_pc2;
    Stocktps4=temps_pc2;
end

if err_als(end)<Stockerr2(end)
    Aalss   =    Aals;Balss  =    Bals;Calss  =    Cals;
    Stockerr2=err_als;
    Stocktps2=temps_als;
end

if err_anls(end)<Stockerr3(end)
    Aanlss  =    Aanls;Banlss  =    Banls;Canlss  =    Canls;
    Stockerr3=err_anls;
    Stocktps3=temps_anls;
end



end

[Apcs,Bpcs,Cpcs] = amb_correct(Apcs,Bpcs,Cpcs,Aanlss,Banlss,Canlss);
[Apcs2,Bpcs2,Cpcs2] = amb_correct(Apcs2,Bpcs2,Cpcs2,Aanlss,Banlss,Canlss);
[Aalss,Balss,Calss] = amb_correct(Apcs,Bpcs,Cpcs,Aanlss,Banlss,Canlss);

%-------------------------------------------------------------------------%
figure
loglog(cumsum(Stocktps1)-1,Stockerr1)
hold on
loglog(cumsum(Stocktps4)-1,Stockerr4)
loglog(cumsum(Stocktps2)-1,Stockerr2)
loglog(cumsum(Stocktps3)-1,Stockerr3)
legend('PROCO-3x3x3','PROCO-21x20x28','ALS','ANLS')
xlabel('time(s)')
ylabel('reconstruction error')


figure
subplot(431)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Apcs)
ylabel('PROCO-3x3x3')
subplot(437)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(abs(Aalss))
ylabel('ALS')
subplot(4,3,10)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Aanlss)
ylabel('ANLS')
xlabel('excitation spectra')
subplot(434)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Apcs2)
ylabel('PROCO-21x20x28')

subplot(432)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Bpcs)
subplot(438)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(abs(Balss))
subplot(4,3,11)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Banlss)
xlabel('emission spectra')
subplot(435)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Bpcs2)

subplot(433)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Cpcs)
subplot(439)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(abs(Calss))
subplot(4,3,12)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Canlss)
xlabel('relative concentrations')
subplot(436)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Cpcs2)
