%% Script for trying some models on coupled EEM NMR LC-MS data

%------------------------------Chemo_simu2 -----------------------------%
% This file tests the Coupled CP ALS algorithms on Data by Acar et al.
%[E. Acar, E. E. Papalexakis, G. Gurdeniz,  M. A. Rasmussen,  A. J. Lawaetz,...
% M. Nilsson,  R. Bro, Structure-Revealing Data Fusion, BMC Bioinformatics, 15: 239, 2014.]
%
% Comments : Compression for fluorescence data

% External functions:
%
% - kr.m              : Khatri-Rao product;
% - als.m             : ALS algorithm;
% - amb_correct_1f.m  : Matching Permutation for two CP models based on 
%                       factors C only;
% - amb_correct.m     : Matching Permutation for two CP models;
% - ccp_als.m         : Coupled CP ALS.
% - Toolboxes         : Tensolab 3.0, Nway toolbox, AO-ADMM, Tensor-toolbox
%
% List of updates                 -     04/02/2016  -     J.E. Cohen 
%                                       Creation of the file
%                                 -     19/08/2016  -     J.E. Cohen
%                                       Added comparison with others
%-------------------------------------------------------------------------%

%--------------Cleaning the workspace and loading functions---------------%
clc;
clear     all;
close     all;
%-------------------------------------------------------------------------%

%load('/Users/cohenjer/Documents/Travail/MATLAB/Data/Data Chemo/data/Multimodal/EEM_NMR_LCMS.mat')
load('D:/Travail/Travail/MATLAB/Data/Data Chemo/data/Multimodal/EEM_NMR_LCMS.mat')

%------------------------ Missing values preprocessing -------------------%

Tx        =    X.data;
Tx        =    permute(Tx,[3,2,1]);

% 3-way block dimensions
dim_X     =     size(Tx);

Tx        =    reshape(Tx,prod(dim_X),1);
temp_x         =     Tx(isnan(Tx));
%temp_miss      =     isnan(Tx);
Tx(isnan(Tx))  =     0;%abs(0.05*randn(size(temp_x)));
Tx             =     reshape(Tx,dim_X);
normTx         =     (Tx(:))'*(Tx(:));
Tx             =     Tx/sqrt(normTx); 
%-----------------------------Model parameters----------------------------%
% Number of components
R_X       =     3;
% Standard deviations of the measurements noise
sigma_n   =     1;
% Number of iterations
iter      =     1000;
% Number of initialization
N    =    1;

%-------------------------------------------------------------------------%

%----------------------- Tucker3 Coupled Compression ---------------------%

Tx_m      =     reshape( permute( Tx , [3,1,2] ) , dim_X(3) , dim_X(2)*dim_X(1) );
dim1c     =     [5,5,5];%[3,3,3];
dim2c     =     [21,20,28];%


[Gx,U,V,W,~,~,Sigma_1]     =     hosvd2(Tx,dim1c,'random');
[Gx2,U2,V2,W2]             =     hosvd(Tx,dim2c,'random');

%figure, plot(diag(Sigma_1),'b')
%figure, plot(log(diag(Sigma_1)),'b')

%------------------------------Computation--------------------------------%

Stockerr1   =     inf*ones(1,iter);
Stockerr2   =     inf*ones(1,iter);
Stockerr3   =     inf*ones(1,iter);
Stockerr4   =     inf*ones(1,iter);
Stockerr5   =     inf*ones(1,iter);
Stockerr6   =     inf*ones(1,iter);

Stocktps1   =     inf*ones(1,iter);
Stocktps2   =     inf*ones(1,iter);
Stocktps3   =     inf*ones(1,iter);
Stocktps4   =     inf*ones(1,iter);
Stocktps5   =     inf*ones(1,iter);
Stocktps6   =     inf*ones(1,iter);

for n=1:N

%Tx        =    reshape(Tx,prod(dim_X),1);
%temp_x         =     Tx(isnan(Tx));
%Tx(isnan(Tx))  =     abs(0.05*randn(size(temp_x)));
%Tx             =     reshape(Tx,dim_X);
%normTx         =     (Tx(:))'*(Tx(:));
%Tx             =     Tx/sqrt(normTx);     
    
%-----------------------------Initialization------------------------------%
% First tensor
A0         =     abs(randn(dim_X(1),R_X));
B0         =     abs(randn(dim_X(2),R_X));
C0         =     abs(randn(dim_X(3),R_X));
A0         =     A0.*repmat(1./sqrt(sum(A0.^2)),dim_X(1),1);
B0         =     B0.*repmat(1./sqrt(sum(B0.^2)),dim_X(2),1); 
C0         =     C0.*repmat(1./sqrt(sum(C0.^2)),dim_X(3),1); 
%-------------------------------------------------------------------------%

%---------------------------Uncoupled ALS---------------------------------%
% FastNNLS
Oldload{1} = A0;Oldload{2} = B0;Oldload{3} = C0;
[Fact,~, ~,~,temps_als,err_als]         =     parafac(Tx,R_X,0,[2 2 2],Oldload);
Aals = Fact{1}; Bals=Fact{2}; Cals=Fact{3};
Cals  = Cals.*repmat(sqrt(sum(Aals.^2)).*sqrt(sum(Bals.^2)),dim_X(3),1);
Aals  = Aals.*repmat(1./sqrt(sum(Aals.^2)),dim_X(1),1);
Bals  = Bals.*repmat(1./sqrt(sum(Bals.^2)),dim_X(2),1);

% ANLS
[Aanls, Banls, Canls, err_anls,temps_anls]    =     anls(Tx,iter,A0,B0,C0); % ANLS

% PROCO-ALS
[Apc,Bpc,Cpc,err_pc,temps_pc,eta]     =     proco_als(Tx,iter,dim1c,'standard','given',A0,B0,C0,0,Gx,U,V,W); % Proco

% CO-ADMM
[Aad, Bad, Cad, err_ad,temps_ad]       =     admm_als(Tx,Gx,U,V,W,iter,5,0.0001,A0,B0,C0);

% AOADMM
ops.init = Oldload;ops.itermax = iter;
[ H, his ] = AOadmm( Tx, R_X, ops );
Apc2 =  H{1}; Bpc2 =  H{2}; Cpc2 = H{3}; err_pc2 = his.err; temps_pc2 = his.time;
Cpc2  = Cpc2.*repmat(sqrt(sum(Apc2.^2)).*sqrt(sum(Bpc2.^2)),dim_X(3),1);
Apc2  = Apc2.*repmat(1./sqrt(sum(Apc2.^2)),dim_X(1),1);
Bpc2  = Bpc2.*repmat(1./sqrt(sum(Bpc2.^2)),dim_X(2),1);


% Tensorlab 3.0
% Structure for structured data fusion
model = struct;
model.variables = Oldload;
model.transform = {1:3, @struct_nonneg};

% Uncompressed
%model.factorizations{1}.data = Tx;

% Compressed
[Us,S] = mlsvd_rsi(Tx, [5 5 5]);
model.factorizations{1}.data = {Us,S};

model.factorizations{1}.cpd = 1:3;
[Htlab, sortie,temps_tlab]                            =     sdf_nls(model, 'Display', 100, 'TolX', 10^(-9), 'TolFun', 10^(-11), 'CGMaxIter', 20);
Atlab = Htlab.factors{1};Btlab  = Htlab.factors{2}; Ctlab  = Htlab.factors{3};
err_tlab = sortie.fval;
finalval = frob(Tx-cpdgen(Htlab.factors))^2;
err_tlab=err_tlab*finalval/err_tlab(end);
Ctlab = Ctlab.*repmat(sqrt(sum(Atlab.^2)).*sqrt(sum(Btlab.^2)),dim_X(3),1);
Atlab = Atlab.*repmat(1./sqrt(sum(Atlab.^2)),dim_X(1),1);
Btlab = Btlab.*repmat(1./sqrt(sum(Btlab.^2)),dim_X(2),1);


% Error Comparaison and keeping best run
if err_pc(end)<Stockerr1(end)
    Apcs   =    Apc;Bpcs   =    Bpc;Cpcs   =    Cpc;
    Stockerr1=err_pc;
    Stocktps1=temps_pc;
end

if err_pc2(end)<Stockerr4(end)
    Apcs2   =    Apc2;Bpcs2   =    Bpc2;Cpcs2   =    Cpc2;
    Stockerr4=err_pc2;
    Stocktps4=temps_pc2;
end

if err_als(end)<Stockerr2(end)
    Aalss   =    Aals;Balss  =    Bals;Calss  =    Cals;
    Stockerr2=err_als;
    Stocktps2=temps_als;
end

if err_anls(end)<Stockerr3(end)
    Aanlss  =    Aanls;Banlss  =    Banls;Canlss  =    Canls;
    Stockerr3=err_anls;
    Stocktps3=temps_anls;
end

if err_ad(end)<Stockerr5(end)
    Aadm  =    Aad;Badm   =    Bad;Cadm   =    Cad;
    Stockerr5=err_ad;
    Stocktps5=temps_ad;
end


if err_tlab(end)<Stockerr6(end)
    Atslab  =    Atlab;Btslab   =    Btlab;Ctslab   =    Ctlab;
    Stockerr6=err_tlab;
    Stocktps6=temps_tlab;
end

end

[Apcs,Bpcs,Cpcs]    = amb_correct(Apcs,Bpcs,Cpcs,Aanlss,Banlss,Canlss);
[Apcs2,Bpcs2,Cpcs2] = amb_correct(Apcs2,Bpcs2,Cpcs2,Aanlss,Banlss,Canlss);
[Aalss,Balss,Calss] = amb_correct(Aalss,Balss,Calss,Aanlss,Banlss,Canlss);
[Aadm,Badm,Cadm]    = amb_correct(Aadm,Badm,Cadm,Aanlss,Banlss,Canlss);
[Atslab,Btslab,Ctslab]    = amb_correct(Atslab,Btslab,Ctslab,Aanlss,Banlss,Canlss);


%-------------------------------------------------------------------------%
figure
loglog(cumsum(Stocktps1)-1,Stockerr1)
hold on
loglog(Stocktps4 ,Stockerr4(2:end))
loglog(cumsum(Stocktps2)  ,Stockerr2)
loglog(cumsum(Stocktps3)-1,Stockerr3)
loglog(cumsum(Stocktps5)-1,Stockerr5)
loglog(Stocktps6,Stockerr6(2:end))
legend('PROCO-3x3x3','AOADMM','FastNNLS','ANLS','CO-ADMM','Tensorlab 3.0')
xlabel('time(s)')
ylabel('reconstruction error')


figure
subplot(631)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Apcs)
ylabel('PROCO-3x3x3')
subplot(637)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(abs(Aalss))
ylabel('FastNNLS')
subplot(6,3,10)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Aanlss)
ylabel('ANLS')
xlabel('excitation spectra')
subplot(634)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Apcs2)
ylabel('AOADMM')
subplot(6,3,13)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Aadm)
ylabel('CO-ADMM')
subplot(6,3,16)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Atslab)
ylabel('Tensorlab')

subplot(632)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Bpcs)
subplot(638)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(abs(Balss))
subplot(6,3,11)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Banlss)
subplot(635)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Bpcs2)
subplot(6,3,14)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Badm)
subplot(6,3,17)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Btslab)
xlabel('emission spectra')

subplot(633)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Cpcs)
subplot(639)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(abs(Calss))
subplot(6,3,12)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Canlss)
subplot(636)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Cpcs2)
subplot(6,3,15)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Cadm)
subplot(6,3,18)
set(gca,'ColorOrder', [0 0 1; 0 1 0; 1 0 0; 0 0.5 0.5; 0.5 0 0.5; 0.5 0.5 0; 0.5 0.5 0.5],'NextPlot', 'replacechildren')
plot(Ctslab)
xlabel('relative concentrations')