%------------------------------Run_Decomp.m-------------------------------%
% In this file, different versions of the compressed algorithms are tested.
% The compressed versions can be compared to the uncompressed projected ALS 
% algorithm. 
% 
% A randomized approximate version of the svd is computed for numerical co-
% -mplexity reasons.
%
% The squared L2 norm of the measurement reconstruction error is evaluated.
%-------------------------------------------------------------------------%

%----------------------------Cleaning variables---------------------------%
clc;
clear all;
close all;

%-----------------------------Model parameters----------------------------%

% --- SIMULATED DATA --- %

% --- PARAMETERS --- %

% CP number of components
R         =     4;
% Uncompressed dimensions
dim_u     =     [100 100 100];
% Compressed dimensions
dim_c     =     [4 4 4];
% Number of iterates for the ALS algorithms
itermax_als     =     100;

SNR = 30;
% CP model and noisy measurements (additive Gaussian noise)

[A,B,C,Y_true,Y,sig]     = tensor_gen(dim_u,R,SNR);

prox_A{1}             =     @(x) bsxfun(@max,x,0); % Nonnegativity
prox_B{1}             =     @(x) bsxfun(@max,x,0); % Nonnegativity
prox_C{1}             =     @(x) bsxfun(@max,x,0); % Nonnegativity


%-------------------------------------------------------------------------%


%----------------------- Compression -------------------------------------%

% Truncated HOSVD --- Replace here by your customized compression
[G,U,V,W,offset] =     hosvd(Y,dim_c,'random'); 

%-----------------------------Initialization------------------------------%
Re =     R; % estimated rank

A0 =     abs(randn(dim_u(1),Re));
B0 =     abs(randn(dim_u(2),Re));
C0 =     abs(randn(dim_u(3),Re));
C0 =     C0.*repmat(1./sqrt(sum(C0.^2)),dim_u(3),1);
A0 =     A0.*repmat(1./sqrt(sum(A0.^2)),dim_u(1),1);
B0 =     B0.*repmat(1./sqrt(sum(B0.^2)),dim_u(2),1);
%-------------------------------------------------------------------------%
   

%-------------------------------------------------------------------------%

%------------------------ Algorithms -------------------------------------%
tic
[Aanls, Banls, Canls, err_anls,time_anls]    =     anls(Y,itermax_als,A0,B0,C0);
toc

[Aals, Bals, Cals, err_als,time_als]         =     als(Y,itermax_als,A0,B0,C0);
%Oldload{1} = A0;Oldload{2} = B0;Oldload{3} = C0;
%[Fact,time_als, err_als,]         =     parafac(Y,R,0,[2 2 2],Oldload);
%Aals = Fact{1}; Bals=Fact{2}; Cals=Fact{3};

[Apc, Bpc, Cpc, err_pc,time_pc]              =     proco_als(Y,itermax_als,dim_c,'standard','given',A0,B0,C0,0,G,U,V,W);

[Aqp, Bqp, Cqp, err_qp,time_qp]              =     qp_als(G,itermax_als,10,dim_c,dim_u,A0,B0,C0,U,V,W,Y);

[Adk, Bdk, Cdk, err_dk,time_dk]              =     dk_als(G,itermax_als,2,dim_c,dim_u,A0,B0,C0,U,V,W,Y);%2

[Aad, Bad, Cad, err_ad,time_ad]              =     admm_als(Y,G,U,V,W,itermax_als,4,0.01,A0,B0,C0);

[Apd1, Bpd1, Cpd1, err_pd1,time_pd1]   =     prox_cp( G,'gradient',itermax_als,5,{U},{V},{W},prox_A,prox_B,prox_C,1.95,0.45,0.01,A0,B0,C0,{zeros(dim_u(1),R)},{zeros(dim_u(2),R)},{zeros(dim_u(3),R)},dim_u,Y);

[Apd2, Bpd2, Cpd2, err_pd2,time_pd2]   =     prox_cp( G,'proximal',itermax_als,5,{U},{V},{W},prox_A,prox_B,prox_C,1.95,0.45,0.01,A0,B0,C0,{zeros(dim_u(1),R)},{zeros(dim_u(2),R)},{zeros(dim_u(3),R)},dim_u,Y);

%[Apd3, Bpd3, Cpd3, err_pd3,time_pd3]   =     prox_cp( G,'all-at-once',itermax_als,5,{U},{V},{W},prox_A,prox_B,prox_C,1.95,0.25,0.1,A0,B0,C0,{zeros(dim_u(1),R)},{zeros(dim_u(2),R)},{zeros(dim_u(3),R)},dim_u,Y);

%--------------------- Corrections ------------------------------------%

[Aals,Bals,Cals]   =amb_correct(Aals ,Bals ,Cals ,A,B,C);
[Aanls,Banls,Canls]=amb_correct(Aanls,Banls,Canls,A,B,C);
[Apc,Bpc,Cpc]=amb_correct(Apc,Bpc,Cpc,A,B,C);
[Aqp,Bqp,Cqp]=amb_correct(Aqp,Bqp,Cqp,A,B,C);
[Adk,Bdk,Cdk]=amb_correct(Adk,Bdk,Cdk,A,B,C);
[Aad, Bad, Cad]=amb_correct(Aad,Bad,Cad,A,B,C);
[Apd1,Bpd1,Cpd1]=amb_correct(Apd1,Bpd1,Cpd1,A,B,C);
[Apd2,Bpd2,Cpd2]=amb_correct(Apd2,Bpd2,Cpd2,A,B,C);
%[Apd3,Bpd3,Cpd3]=amb_correct(Apd3,Bpd3,Cpd3,A,B,C);
  
%% ---------------------   Plots   ------------------------------%


figure

% subplot(9,2,1)
% plot(Cals(:,1))
% hold on
% plot(C(:,1))
% subplot(9,2,3)
% plot(Canls(:,1))
% hold on
% plot(C(:,1))
% subplot(9,2,5)
% plot(Cpc(:,1))
% hold on
% plot(C(:,1))
% subplot(9,2,7)
% plot(Cqp(:,1))
% hold on
% plot(C(:,1))
% subplot(9,2,9)
% plot(Cdk(:,1))
% hold on
% plot(C(:,1))
% subplot(9,2,11)
% plot(Cpd1(:,1))
% hold on
% plot(C(:,1))
% subplot(9,2,13)
% plot(Cpd2(:,1))
% hold on
% plot(C(:,1))
% subplot(9,2,15)
% plot(Cpd3(:,1))
% hold on
% plot(C(:,1))

% If error is co;puted at each run
%time
%subplot(9,2,[2,4,6,8])
subplot(211)
loglog(cumsum(time_als)-1 ,err_als   ,'b')    %blue for ALS
hold on
loglog(cumsum(time_anls)-1,err_anls  ,'-r')   %red for NALS
hold on
loglog(cumsum(time_pc)-1  ,err_pc    ,'-g')   %green for PROCO
hold on
loglog(cumsum(time_qp)-1  ,err_qp    ,'-k')   %black for QP active set
hold on
loglog(cumsum(time_dk)-1  ,err_dk    ,'-m')   %magenta for Dykstra
hold on
loglog(cumsum(time_ad)-1  ,err_ad    ,'--k')   %dotted black for ADMM
hold on
loglog(cumsum(time_pd1)-1 ,err_pd1   ,'-c')   %cyan for PRIMAL DUAL 1
hold on
loglog(cumsum(time_pd2)-1 ,err_pd2   ,'-y')  %yellow for PRIMAL DUAL 2
hold on
%loglog(cumsum(time_pd3)-1 ,err_pd3   ,'--m') %magenta for PRIMAL DUAL 3
%axis([0 0.5 10^-3 10^2])
xlabel('time (s)')
ylabel('Reconstruction Error')

%iterations
%subplot(9,2,[10,12,14,16])
subplot(212)
loglog(err_als   ,'b') %blue for ALS
hold on
loglog(err_anls  ,'-r') %red for NALS
hold on
loglog(err_pc    ,'-g') %green for PROCO
hold on
loglog(err_qp    ,'-k') %black for QP active set
hold on
loglog(err_dk    ,'-m') %magenta for Dykstra
hold on
loglog(err_ad    ,'--k') %dotted black for ADMM
hold on
loglog(err_pd1   ,'-c') %cyan for PRIMAL DUAL 1
hold on
loglog(err_pd2   ,'-y') %yellow for PRIMAL DUAL 2
hold on
%loglog(err_pd3   ,'--m') %magentafor PRIMAL DUAL 3
xlabel('iterations')
ylabel('Reconstruction Error')
legend('ALS','NALS','PROCO','Active Set','Dykstra','COADMM','GradientPD','ProxPD')%,'ALLPD')