%------------------------------Run_Decomp2.m-------------------------------%
% In this file, different versions of the compressed algorithms are tested.
% The compressed versions can be compared to the uncompressed projected ALS 
% algorithm. 
% 
% A randomized approximate version of the svd is computed for numerical co-
% -mplexity reasons.
%
% The squared L2 norm of the measurement reconstruction error is evaluated.
%-------------------------------------------------------------------------%

%----------------------------Cleaning variables---------------------------%
clc;
clear all;
close all;

%-----------------------------Model parameters----------------------------%

% --- SIMULATED DATA --- %

% --- PARAMETERS --- %

% CP number of components
R         =     5;
% Uncompressed dimensions
dim_u     =     [100 100 100];
% Compressed dimensions
dim_c     =     [R R R];
% Number of iterates for the ALS algorithms
itermax_als     =     200;

SNR = 30;
% CP model and noisy measurements (additive Gaussian noise)

[A,B,C,Y_true,Y,sig]     = tensor_gen(dim_u,R,SNR,'yes'); % put 'yes' for illed-conditionned

% Stock variables
Nmax = 10;

err_annls_i  = zeros(Nmax,itermax_als+1);
err_anls_i   = zeros(Nmax,itermax_als+1);
err_pc_i     = zeros(Nmax,itermax_als+1);
err_qp_i     = zeros(Nmax,itermax_als+1);
err_ad_i     = zeros(Nmax,itermax_als+1);
err_tlab_i   = zeros(Nmax,251);

time_annls_i  = zeros(Nmax,itermax_als+1);
time_anls_i   = zeros(Nmax,itermax_als+1);
time_pc_i     = zeros(Nmax,itermax_als+1);
time_qp_i     = zeros(Nmax,itermax_als);%+1 pour pers
time_ad_i     = zeros(Nmax,itermax_als+1);
time_tlab_i   = zeros(Nmax,250);


%-------------------------------------------------------------------------%


%----------------------- Compression -------------------------------------%

% Truncated HOSVD --- Replace here by your customized compression
[G,U,V,W,offset] =     hosvd(Y,dim_c,'random'); 

for i=1:Nmax

%-----------------------------Initialization------------------------------%
Re =     R; % estimated rank

A0 =     abs(randn(dim_u(1),Re));
B0 =     abs(randn(dim_u(2),Re));
C0 =     abs(randn(dim_u(3),Re));
C0 =     C0.*repmat(1./sqrt(sum(C0.^2)),dim_u(3),1);
A0 =     A0.*repmat(1./sqrt(sum(A0.^2)),dim_u(1),1);
B0 =     B0.*repmat(1./sqrt(sum(B0.^2)),dim_u(2),1);
%-------------------------------------------------------------------------%
   

%-------------------------------------------------------------------------%

%------------------------ Algorithms -------------------------------------%
tic
[Aanls, Banls, Canls, err_anls,time_anls]    =     anls(Y,itermax_als,A0,B0,C0);
toc

%[Aannls, Bannls, Cannls, err_annls,time_annls]         =     als(Y,itermax_als,A0,B0,C0);
Oldload{1} = A0;Oldload{2} = B0;Oldload{3} = C0;
[Fact,~, ~,~,time_annls,err_annls]         =     parafac(Y,Re,0,[2 2 2],Oldload);
Aannls = Fact{1}; Bannls=Fact{2}; Cannls=Fact{3};
err_annls =[err_annls,ones(1,itermax_als+1-size(err_annls,2))*err_annls(end)];
time_annls=[time_annls,zeros(1,itermax_als+1-size(time_annls,2))];


[Apc, Bpc, Cpc, err_pc,time_pc]              =     proco_als(Y,itermax_als,dim_c,'standard','given',A0,B0,C0,0,G,U,V,W);

%[Aqp, Bqp, Cqp, err_qp,time_qp]              =     admm_als_full(Y,itermax_als,5,0.01,A0,B0,C0);

init_cell{1} = A0; init_cell{2} = B0; init_cell{3} = C0;
ops.init = Oldload;ops.itermax = itermax_als;
[ H, his ] = AOadmm( Y, Re, ops );
Aqp  =  H{1}; Bqp  =  H{2}; Cqp = H{3}; err_qp = his.err; time_qp = his.time;
err_qp =[err_qp,ones(1,itermax_als+1-size(err_qp,2))*err_qp(end)];
time_qp=[time_qp,ones(1,itermax_als-size(time_qp,2))*time_qp(end)];
Cqp  = Cqp.*repmat(sqrt(sum(Aqp.^2)).*sqrt(sum(Bqp.^2)),dim_u(3),1);
Aqp  = Aqp.*repmat(1./sqrt(sum(Aqp.^2)),dim_u(1),1);
Bqp  = Bqp.*repmat(1./sqrt(sum(Bqp.^2)),dim_u(2),1);


[Aad, Bad, Cad, err_ad,time_ad]              =     admm_als(Y,G,U,V,W,itermax_als,5,0.01,A0,B0,C0);

% Structure for structured data fusion
model = struct;
model.variables = Oldload;
model.transform = {1:3, @struct_nonneg};

% Uncompressed
model.factorizations{1}.data = Y;

% Compressed
%[Us,S] = mlsvd_rsi(Y, [R R R]);
%model.factorizations{1}.data = {Us,S};

model.factorizations{1}.cpd = 1:3;
[Htlab, sortie,time_tlab]                            =     sdf_nls(model, 'Display', 100, 'TolX', 10^(-16), 'TolFun', 10^(-32), 'CGMaxIter', 75,'MaxIter',250);
Atlab = Htlab.factors{1};Btlab  = Htlab.factors{2}; Ctlab  = Htlab.factors{3};err_tlab = sortie.fval;
finalval = frob(Y-cpdgen(Htlab.factors))^2;
err_tlab=err_tlab*finalval/err_tlab(end);
err_tlab =[err_tlab,ones(1,251-size(err_tlab,2))*err_tlab(end)];
time_tlab=[time_tlab,ones(1,250-size(time_tlab,2))*time_tlab(end)];
Ctlab = Ctlab.*repmat(sqrt(sum(Atlab.^2)).*sqrt(sum(Btlab.^2)),dim_u(3),1);
Atlab = Atlab.*repmat(1./sqrt(sum(Atlab.^2)),dim_u(1),1);
Btlab = Btlab.*repmat(1./sqrt(sum(Btlab.^2)),dim_u(2),1);

%--------------------- Corrections ------------------------------------%

[Aannls,Bannls,Cannls]   =amb_correct(Aannls ,Bannls ,Cannls ,A,B,C);
[Aanls,Banls,Canls]=amb_correct(Aanls,Banls,Canls,A,B,C);
[Apc,Bpc,Cpc]=amb_correct(Apc,Bpc,Cpc,A,B,C);
[Aqp,Bqp,Cqp]=amb_correct(Aqp,Bqp,Cqp,A,B,C);
%[Aao,Bao,Cao]=amb_correct(Aao,Bao,Cao,A,B,C);
[Aad, Bad, Cad]=amb_correct(Aad,Bad,Cad,A,B,C);
[Atlab, Btlab, Ctlab]=amb_correct(Atlab,Btlab,Ctlab,A,B,C);


fac_annls(i) = sqrt(sum(sum((Aannls(:,1:R)-A).^2))) + sqrt(sum(sum((Bannls(:,1:R)-B).^2))) + sqrt(sum(sum((Cannls(:,1:R)-C).^2)));
fac_anls(i)  = sqrt(sum(sum((Aanls(:,1:R)-A).^2)))  + sqrt(sum(sum((Banls(:,1:R)-B).^2)))  + sqrt(sum(sum((Canls(:,1:R)-C).^2)));
fac_pc(i)    = sqrt(sum(sum((Apc(:,1:R)-A).^2)))    + sqrt(sum(sum((Bpc(:,1:R)-B).^2)))    + sqrt(sum(sum((Cpc(:,1:R)-C).^2)));
fac_qp(i)    = sqrt(sum(sum((Aqp(:,1:R)-A).^2)))    + sqrt(sum(sum((Bqp(:,1:R)-B).^2)))    + sqrt(sum(sum((Cqp(:,1:R)-C).^2)));
fac_ad(i)    = sqrt(sum(sum((Aad(:,1:R)-A).^2)))    + sqrt(sum(sum((Bad(:,1:R)-B).^2)))    + sqrt(sum(sum((Cad(:,1:R)-C).^2)));
fac_tlab(i)  = sqrt(sum(sum((Atlab(:,1:R)-A).^2)))  + sqrt(sum(sum((Btlab(:,1:R)-B).^2)))  + sqrt(sum(sum((Ctlab(:,1:R)-C).^2)));


time_annls_i(i,1:size(time_annls,2)) = time_annls;
time_anls_i(i,1:size(time_anls,2)) = time_anls;
time_pc_i(i,1:size(time_pc,2)) = time_pc;
time_qp_i(i,1:size(time_qp,2)) = time_qp;
time_ad_i(i,1:size(time_ad,2)) = time_ad;
time_tlab_i(i,1:size(time_tlab,2)) = time_tlab;

err_annls_i(i,:) = err_annls;
err_anls_i(i,:) = err_anls;
err_pc_i(i,:) = err_pc;
err_qp_i(i,:) = err_qp;
err_ad_i(i,:) = err_ad;
err_tlab_i(i,:) = err_tlab;

end

%% ---------------------   Plots   ------------------------------%

% NOTE : Removed 47 of QP in S3 

scores_fac = [fac_annls',fac_anls',fac_pc',fac_qp',fac_ad',fac_tlab'];

time_annls = mean(time_annls_i);
time_anls  = mean(time_anls_i);
time_pc    = mean(time_pc_i);
time_qp    = mean(time_qp_i);
time_ad    = mean(time_ad_i);
time_tlab  = mean(time_tlab_i);

err_annls = mean(err_annls_i);
err_anls  = mean(err_anls_i);
err_pc    = mean(err_pc_i);
err_qp    = mean(err_qp_i);
err_ad    = mean(err_ad_i);
err_tlab  = mean(err_tlab_i);



% If error is co;puted at each run
%time
%subplot(9,2,[2,4,6,8])
%subplot(211)
figure
loglog(cumsum(time_annls) ,err_annls   ,'b')    %blue for NWAY
hold on
loglog(cumsum(time_anls)-1,err_anls  ,'-r')   %red for NALS
hold on
loglog(cumsum(time_pc)-1  ,err_pc    ,'-g')   %green for PROCO
hold on
%loglog(cumsum(time_qp)-1  ,err_qp    ,'-k')   %black for ADMM full %remove 2 end for perso
%hold on
loglog([0,time_qp]  ,err_qp    ,'-k')   %magenta for Dykstra
hold on
loglog(cumsum(time_ad)-1  ,err_ad    ,'--k')   %dotted black for ADMM 
hold on
loglog([0,time_tlab(time_tlab>0)],err_tlab(err_tlab>0),'-c')
xlabel('time (s)')
ylabel('Reconstruction Error')
legend('FastNNLS','NALS','PROCO','AOADMM','COADMM','Tensorlab')%'AOADMM'

figure
boxplot(scores_fac,'labels',{'FastNNLS','NALS','PROCO','AOADMM','COADMM','Tensorlab'})