%------------------------------Simulations.m-------------------------------%
% In this file, different versions of the compressed algorithms are tested.
% The compressed versions can be compared to the uncompressed projected ALS 
% algorithm. 
% 
% A randomized approximate version of the svd is computed for numerical co-
% -mplexity reasons.
%
% The squared L2 norm of the measurement reconstruction error is evaluated.
%-------------------------------------------------------------------------%

%----------------------------Cleaning variables---------------------------%
clc;
clear all;
close all;

%-----------------------------Model parameters----------------------------%

% --- SIMULATED DATA --- %

% --- PARAMETERS --- %

% Number of initializations
N         =     1;
% Number of averaged results
M         =     1;
% Number of iterates for the ALS algorithms
itermax_als     =     25;
% Storage variables
Stockdist =     ones(M,itermax_als+1);

% CP number of components
R         =     4;
% Uncompressed dimensions
dim_u     =     [100 100 100];
% Compressed dimensions
dim_c     =     [5 5 5];


for i=1:M

SNR       =     10;
% CP model and noisy measurements (additive Gaussian noise)

[A,B,C,Y_true,Y,sig]     = tensor_gen(dim_u,R,SNR);


%-------------------------------------------------------------------------%


%----------------------- Compression -------------------------------------%

% Truncated HOSVD --- Replace here by your customized compression
[G,U,V,W,~] =     hosvd(Y,dim_c,'random'); 

%-----------------------------Initialization------------------------------%
Re =     R; % estimated rank

for n=1:N

A0 =     abs(randn(dim_u(1),Re));
B0 =     abs(randn(dim_u(2),Re));
C0 =     abs(randn(dim_u(3),Re));
C0 =     C0.*repmat(1./sqrt(sum(C0.^2)),dim_u(3),1);
A0 =     A0.*repmat(1./sqrt(sum(A0.^2)),dim_u(1),1);
B0 =     B0.*repmat(1./sqrt(sum(B0.^2)),dim_u(2),1);
%-------------------------------------------------------------------------%
   

%-------------------------------------------------------------------------%

%------------------------ Algorithms -------------------------------------%

[Apc, Bpc, Cpc, err_pc,temps_pc,eta]         =     proco_als(Y,itermax_als,dim_c,'standard','given',A0,B0,C0,0,G,U,V,W);

%--------------------- Corrections ------------------------------------%

[Apc,Bpc,Cpc]=amb_correct(Apc,Bpc,Cpc,A,B,C);



end % init
end % averages
% ---------------------   Error values   ------------------------------%

