%------------------------------Simulations.m-------------------------------%
% In this file, different versions of the compressed algorithms are tested.
% The compressed versions can be compared to the uncompressed projected ALS 
% algorithm. 
% 
% A randomized approximate version of the svd is computed for numerical co-
% -mplexity reasons.
%
% The squared L2 norm of the measurement reconstruction error is evaluated.
%-------------------------------------------------------------------------%

%----------------------------Cleaning variables---------------------------%
clc;
clear all;
close all;

%-----------------------------Model parameters----------------------------%

% --- SIMULATED DATA --- %

% --- PARAMETERS --- %

% Number of initializations
N         =     2;
% Number of averaged results
M         =     30;
% Number of iterates for the ALS algorithms
itermax_als     =     25;
% Storage variables
Stockerr1   =     ones(M,4);
Stockerr2   =     ones(M,4);
Stockerr3   =     ones(M,4);
Stockerr4   =     ones(M,4);
Stockerr5   =     ones(M,4);
Stockerr6   =     ones(M,4);

% CP number of components
R         =     4;
% Uncompressed dimensions
dim_u     =     [100 100 100];
% Compressed dimensions
dim_c1    =     [4 4 4];
dim_c2    =     [5 5 5];
dim_c3    =     [7 7 7];
dim_c4    =     [10 10 10];
dim_c5    =     [20 20 20];


for i=1:M

SNR       =     10;
% CP model and noisy measurements (additive Gaussian noise)

[A,B,C,Y_true,Y,sig]     = tensor_gen(dim_u,R,SNR);


%-------------------------------------------------------------------------%


%----------------------- Compression -------------------------------------%

% Truncated HOSVD --- Replace here by your customized compression
[G1,U1,V1,W1,~] =     hosvd(Y,dim_c1,'random'); 
[G2,U2,V2,W2,~] =     hosvd(Y,dim_c2,'random'); 
[G3,U3,V3,W3,~] =     hosvd(Y,dim_c3,'random'); 
[G4,U4,V4,W4,~] =     hosvd(Y,dim_c4,'random'); 
[G5,U5,V5,W5,~] =     hosvd(Y,dim_c5,'random'); 

%-----------------------------Initialization------------------------------%
Re =     R; % estimated rank

for n=1:N

A0 =     abs(randn(dim_u(1),Re));
B0 =     abs(randn(dim_u(2),Re));
C0 =     abs(randn(dim_u(3),Re));
C0 =     C0.*repmat(1./sqrt(sum(C0.^2)),dim_u(3),1);
A0 =     A0.*repmat(1./sqrt(sum(A0.^2)),dim_u(1),1);
B0 =     B0.*repmat(1./sqrt(sum(B0.^2)),dim_u(2),1);
%-------------------------------------------------------------------------%
   

%-------------------------------------------------------------------------%

%------------------------ Algorithms -------------------------------------%

[Aanls, Banls, Canls, err_anls,temps_anls]    =     anls(Y,itermax_als,A0,B0,C0);

[Apc2, Bpc2, Cpc2, err_pc2,temps_pc2]         =     proco_als(Y,itermax_als,dim_c2,'standard','given',A0,B0,C0,0,G2,U2,V2,W2);
[Apc5, Bpc5, Cpc5, err_pc5,temps_pc5]         =     proco_als(Y,itermax_als,dim_c5,'standard','given',A0,B0,C0,0,G5,U5,V5,W5);
[Apc1, Bpc1, Cpc1, err_pc1,temps_pc1]         =     proco_als(Y,itermax_als,dim_c1,'standard','given',A0,B0,C0,0,G1,U1,V1,W1);
[Apc3, Bpc3, Cpc3, err_pc3,temps_pc3]         =     proco_als(Y,itermax_als,dim_c3,'standard','given',A0,B0,C0,0,G3,U3,V3,W3);
[Apc4, Bpc4, Cpc4, err_pc4,temps_pc4]         =     proco_als(Y,itermax_als,dim_c4,'standard','given',A0,B0,C0,0,G4,U4,V4,W4);

%--------------------- Corrections ------------------------------------%

[Aanls,Banls,Canls]=amb_correct(Aanls,Banls,Canls,A,B,C);
[Apc1,Bpc1,Cpc1]=amb_correct(Apc1,Bpc1,Cpc1,A,B,C);
[Apc2,Bpc2,Cpc2]=amb_correct(Apc2,Bpc2,Cpc2,A,B,C);
[Apc3,Bpc3,Cpc3]=amb_correct(Apc3,Bpc3,Cpc3,A,B,C);
[Apc4,Bpc4,Cpc4]=amb_correct(Apc4,Bpc4,Cpc4,A,B,C);
[Apc5,Bpc5,Cpc5]=amb_correct(Apc5,Bpc5,Cpc5,A,B,C);


% Error Comparaison and keeping best run
if err_pc1(end)<Stockerr1(i,4)
    Stockerr1(i,4)=err_pc1(end);Stockerr1(i,1) = sum(sum((A-Apc1).^2));Stockerr1(i,2) = sum(sum((B-Bpc1).^2));Stockerr1(i,3) = sum(sum((C-Cpc1).^2));
end

if err_pc2(end)<Stockerr2(i,4)
    Stockerr2(i,4)=err_pc2(end);Stockerr2(i,1) = sum(sum((A-Apc2).^2));Stockerr2(i,2) = sum(sum((B-Bpc2).^2));Stockerr2(i,3) = sum(sum((C-Cpc2).^2));
end

if err_pc3(end)<Stockerr3(i,4)
    Stockerr3(i,4)=err_pc3(end);Stockerr3(i,1) = sum(sum((A-Apc3).^2));Stockerr3(i,2) = sum(sum((B-Bpc3).^2));Stockerr3(i,3) = sum(sum((C-Cpc3).^2));
end

if err_pc4(end)<Stockerr4(i,4)
    Stockerr4(i,4)=err_pc4(end);Stockerr4(i,1) = sum(sum((A-Apc4).^2));Stockerr4(i,2) = sum(sum((B-Bpc4).^2));Stockerr4(i,3) = sum(sum((C-Cpc4).^2));
end
if err_pc5(end)<Stockerr5(i,4)
    Stockerr5(i,4)=err_pc5(end);Stockerr5(i,1) = sum(sum((A-Apc5).^2));Stockerr5(i,2) = sum(sum((B-Bpc5).^2));Stockerr5(i,3) = sum(sum((C-Cpc5).^2));
end

if err_anls(end)<Stockerr6(i,4)
    Stockerr6(i,4)=err_anls(end);Stockerr6(i,1) = sum(sum((A-Aanls).^2));Stockerr6(i,2) = sum(sum((B-Banls).^2));Stockerr6(i,3) = sum(sum((C-Canls).^2));
end


end % init
end % averages
% ---------------------   Error values   ------------------------------%


synthese(1,:) = mean(Stockerr1,1) ;
synthese(2,:) = mean(Stockerr2,1) ;
synthese(3,:) = mean(Stockerr3,1) ;
synthese(4,:) = mean(Stockerr4,1) ;
synthese(5,:) = mean(Stockerr5,1) ;
synthese(6,:) = mean(Stockerr6,1) ;

