%------------------------------Simulations.m-------------------------------%
% In this file, different versions of the compressed algorithms are tested.
% The compressed versions can be compared to the uncompressed projected ALS 
% algorithm. 
% 
% A randomized approximate version of the svd is computed for numerical co-
% -mplexity reasons.
%
% The squared L2 norm of the measurement reconstruction error is evaluated.
%-------------------------------------------------------------------------%

%----------------------------Cleaning variables---------------------------%
clc;
clear all;
close all;

%-----------------------------Model parameters----------------------------%

% --- SIMULATED DATA --- %

% --- PARAMETERS --- %

% Number of initializations
N         =     4;
% Number of averaged results
M         =     30;
% Number of iterates for the ALS algorithms
itermax_als     =     25;
% Storage variables
Stockerr1   =     inf*ones(M,itermax_als+1);
Stockerr2   =     inf*ones(M,itermax_als+1);
Stockerr3   =     inf*ones(M,itermax_als+1);
Stockerr4   =     inf*ones(M,itermax_als+1);
Stockerr5   =     inf*ones(M,itermax_als+1);
Stockerr6   =     inf*ones(M,itermax_als+1);
Stocktps1   =     inf*ones(M,itermax_als+1);
Stocktps2   =     inf*ones(M,itermax_als+1);
Stocktps3   =     inf*ones(M,itermax_als+1);
Stocktps4   =     inf*ones(M,itermax_als+1);
Stocktps5   =     inf*ones(M,itermax_als+1);
Stocktps6   =     inf*ones(M,itermax_als+1);
% CP number of components
R         =     4;
% Uncompressed dimensions
dim_u     =     [100 100 100];
% Compressed dimensions
dim_c1    =     [4 4 4];
dim_c2    =     [5 5 5];
dim_c3    =     [7 7 7];
dim_c4    =     [10 10 10];
dim_c5    =     [20 20 20];


for i=1:M

SNR       =     30;
% CP model and noisy measurements (additive Gaussian noise)

[A,B,C,Y_true,Y,sig]     = tensor_gen(dim_u,R,SNR);


%-------------------------------------------------------------------------%


%----------------------- Compression -------------------------------------%

% Truncated HOSVD --- Replace here by your customized compression
[G1,U1,V1,W1,~] =     hosvd(Y,dim_c1,'random'); 
[G2,U2,V2,W2,~] =     hosvd(Y,dim_c2,'random'); 
[G3,U3,V3,W3,~] =     hosvd(Y,dim_c3,'random'); 
[G4,U4,V4,W4,~] =     hosvd(Y,dim_c4,'random'); 
[G5,U5,V5,W5,~] =     hosvd(Y,dim_c5,'random'); 

%-----------------------------Initialization------------------------------%
Re =     R; % estimated rank

for n=1:N

A0 =     abs(randn(dim_u(1),Re));
B0 =     abs(randn(dim_u(2),Re));
C0 =     abs(randn(dim_u(3),Re));
C0 =     C0.*repmat(1./sqrt(sum(C0.^2)),dim_u(3),1);
A0 =     A0.*repmat(1./sqrt(sum(A0.^2)),dim_u(1),1);
B0 =     B0.*repmat(1./sqrt(sum(B0.^2)),dim_u(2),1);
%-------------------------------------------------------------------------%
   

%-------------------------------------------------------------------------%

%------------------------ Algorithms -------------------------------------%

[Aanls, Banls, Canls, err_anls,temps_anls]    =     anls(Y,itermax_als,A0,B0,C0);

[Apc2, Bpc2, Cpc2, err_pc2,temps_pc2]         =     proco_als(Y,itermax_als,dim_c2,'standard','given',A0,B0,C0,0,G2,U2,V2,W2);
[Apc5, Bpc5, Cpc5, err_pc5,temps_pc5]         =     proco_als(Y,itermax_als,dim_c5,'standard','given',A0,B0,C0,0,G5,U5,V5,W5);
[Apc1, Bpc1, Cpc1, err_pc1,temps_pc1]         =     proco_als(Y,itermax_als,dim_c1,'standard','given',A0,B0,C0,0,G1,U1,V1,W1);
[Apc3, Bpc3, Cpc3, err_pc3,temps_pc3]         =     proco_als(Y,itermax_als,dim_c3,'standard','given',A0,B0,C0,0,G3,U3,V3,W3);
[Apc4, Bpc4, Cpc4, err_pc4,temps_pc4]         =     proco_als(Y,itermax_als,dim_c4,'standard','given',A0,B0,C0,0,G4,U4,V4,W4);

%--------------------- Corrections ------------------------------------%

[Aanls,Banls,Canls]=amb_correct(Aanls,Banls,Canls,A,B,C);
[Apc1,Bpc1,Cpc1]=amb_correct(Apc1,Bpc1,Cpc1,A,B,C);
[Apc2,Bpc2,Cpc2]=amb_correct(Apc2,Bpc2,Cpc2,A,B,C);
[Apc3,Bpc3,Cpc3]=amb_correct(Apc3,Bpc3,Cpc3,A,B,C);
[Apc4,Bpc4,Cpc4]=amb_correct(Apc4,Bpc4,Cpc4,A,B,C);
[Apc5,Bpc5,Cpc5]=amb_correct(Apc5,Bpc5,Cpc5,A,B,C);


% Error Comparaison and keeping best run
if err_pc1(end)<Stockerr1(i,end)
    Apcs1   =    Apc1;Bpcs1   =    Bpc1;Cpcs1   =    Cpc1;
    Stockerr1(i,:)=err_pc1;
    Stocktps1(i,:)=temps_pc1;
end

if err_pc2(end)<Stockerr2(i,end)
    Apcs2   =    Apc2;Bpcs2   =    Bpc2;Cpcs2   =    Cpc2;
    Stockerr2(i,:)=err_pc2;
    Stocktps2(i,:)=temps_pc2;
end

if err_pc3(end)<Stockerr3(i,end)
    Apcs3   =    Apc3;Bpcs3   =    Bpc3;Cpcs3   =    Cpc3;
    Stockerr3(i,:)=err_pc3;
    Stocktps3(i,:)=temps_pc3;
end

if err_pc4(end)<Stockerr4(i,end)
    Apcs4   =    Apc4;Bpcs4   =    Bpc4;Cpcs4   =    Cpc4;
    Stockerr4(i,:)=err_pc4;
    Stocktps4(i,:)=temps_pc4;
end
if err_pc5(end)<Stockerr5(i,end)
    Apcs5   =    Apc5;Bpcs5   =    Bpc5;Cpcs5   =    Cpc5;
    Stockerr5(i,:)=err_pc5;
    Stocktps5(i,:)=temps_pc5;
end

if err_anls(end)<Stockerr6(i,end)
    Aanlss  =    Aanls;Banlss  =    Banls;Canlss  =    Canls;
    Stockerr6(i,:)=err_anls;
    Stocktps6(i,:)=temps_anls;
end


end % init
end % averages
% ---------------------   Plots   ------------------------------%

% best run plot
figure(1)


% If error is computed at each run
%time
%subplot(9,2,[2,4,6,8])
subplot(211)
loglog(cumsum(Stocktps6(1,:))-1  ,Stockerr6(1,:)  ,'-r')   %red for NALS
hold on
loglog(cumsum(Stocktps1(1,:))-1  ,Stockerr1(1,:)   ,'-g')   %green
hold on
loglog(cumsum(Stocktps2(1,:))-1  ,Stockerr2(1,:)  ,'-k')   %black 
hold on
loglog(cumsum(Stocktps3(1,:))-1  ,Stockerr3(1,:)  ,'-m')   %magenta
hold on
loglog(cumsum(Stocktps4(1,:))-1  ,Stockerr4(1,:)  ,'-c')   %cyan 
hold on
loglog(cumsum(Stocktps5(1,:))-1  ,Stockerr5(1,:)  ,'-b')  %blue
hold on
%axis([0 0.5 10^-3 10^2])
xlabel('time (s)')
ylabel('Reconstruction Error')

%iterations
%subplot(9,2,[10,12,14,16])
subplot(212)
loglog(Stockerr6(1,:)  ,'-r') %red for NALS
hold on
loglog(Stockerr1(1,:)    ,'-g') %green
hold on
loglog(Stockerr2(1,:)   ,'-k') %black 
hold on
loglog(Stockerr3(1,:)   ,'-m') %magenta
hold on
loglog(Stockerr4(1,:)   ,'-c') %cyan
hold on
loglog(Stockerr5(1,:)   ,'-b') %blue
hold on
xlabel('iterations')
ylabel('Reconstruction Error')
legend('ANLS','PROCO1','PROCO2','PROCO3','PROCO4','PROCO5')

% Averaged plot
figure(2)

Stockerr1f   =     mean(Stockerr1,1);
Stockerr2f   =     mean(Stockerr2,1);
Stockerr3f  =     mean(Stockerr3,1);
Stockerr4f   =     mean(Stockerr4,1);
Stockerr5f   =     mean(Stockerr5,1);
Stockerr6f   =     mean(Stockerr6,1);
Stocktps1f   =     mean(Stocktps1,1);
Stocktps2f   =     mean(Stocktps2,1);
Stocktps3f   =     mean(Stocktps3,1);
Stocktps4f   =     mean(Stocktps4,1);
Stocktps5f   =     mean(Stocktps5,1);
Stocktps6f   =     mean(Stocktps6,1);

% If error is computed at each run
%time
%subplot(9,2,[2,4,6,8])
subplot(211)
loglog(cumsum(Stocktps6f)-1  ,Stockerr6f  ,'-r')   %red for NALS
hold on
loglog(cumsum(Stocktps1f)-1  ,Stockerr1f   ,'-g')   %green
hold on
loglog(cumsum(Stocktps2f)-1  ,Stockerr2f  ,'-k')   %black 
hold on
loglog(cumsum(Stocktps3f)-1  ,Stockerr3f  ,'-m')   %magenta
hold on
loglog(cumsum(Stocktps4f)-1  ,Stockerr4f  ,'-c')   %cyan 
hold on
loglog(cumsum(Stocktps5f)-1  ,Stockerr5f  ,'-b')  %blue
hold on
%axis([0 0.5 10^-3 10^2])
xlabel('time (s)')
ylabel('Reconstruction Error')

%iterations
%subplot(9,2,[10,12,14,16])
subplot(212)
loglog(Stockerr6f  ,'-r') %red for NALS
hold on
loglog(Stockerr1f    ,'-g') %green
hold on
loglog(Stockerr2f   ,'-k') %black 
hold on
loglog(Stockerr3f   ,'-m') %magenta
hold on
loglog(Stockerr4f   ,'-c') %cyan
hold on
loglog(Stockerr5f   ,'-b') %blue
hold on
xlabel('iterations')
ylabel('Reconstruction Error')
legend('ANLS','PROCO1','PROCO2','PROCO3','PROCO4','PROCO5')
