function  [A,B,C,X,Y,sig] = tensor_gen(dim,R,SNR,cond)
%-------------------------------------------------------------------------%
% [A,B,C,X,Y] = tensor_gen(dim,R,sigma)
%
% Random CP model generation. All factors are nonnegative. Factors A and B
% have columns with unitary L1 norm. The measured 3-way block is corrupted
% by additive i.i.d. Gaussian noise.
%
% The inputs are:
% 
% - dim         : tensor dimensions.
% - R           : rank.
% - SNR         : desired signal to noise ratio.
% - cond        : 'yes' for ill-conditionned.
%
% normalized lines.
%
% The outputs are:
% 
% - A           : first factor.
% - B           : second factor.
% - C           : third factor.
% - X           : tensor without noise.
% - Y           : tensor with noise.
% - sig         : noise variance
%
% List of updates                 -     15/05/2014  -     Rodrigo Cabral
%                                       Creation of the file 
%                                 -     06/04/2016  -     J.E.Cohen
%                                       Adaptation for manuscript 
%-------------------------------------------------------------------------%

%--------------------------Tensor generation------------------------------%
% Unnormalized nonnegative factors
A   =     abs(randn(dim(1),R));
B   =     abs(randn(dim(2),R));
C   =     abs(randn(dim(3),R));    

if strcmp(cond,'yes')
   A(:,2) =     0.95*A(:,1) + 0.05*A(:,2); 
end

% Normalized nonnegative factors
A         =     A./ repmat(sqrt(sum(A.^2)), dim(1), 1);
B         =     B./ repmat(sqrt(sum(B.^2)), dim(2), 1);
C         =     C./ repmat(sqrt(sum(C.^2)), dim(3), 1);

% First mode tensor without noise
X_m1=     A*kr(C,B)';
% Tensor without noise
X   =     reshape(X_m1,dim);
% Tensor measurement
sig =     sqrt(R/dim(1)/dim(2)/dim(3)*10^-(SNR/10));
Y   =     X+sig*randn(dim);
%-------------------------------------------------------------------------%